"use strict";

// TEST, for now.
// Handles 
function mdImgImporter() {
	this.test = "";
}

// Convert various strings to url or dataUrl.
// e.g. "data:image/...", "<img src='...'/>", "<html><body><img src='...'/></body></html>".
mdImgImporter.prototype._imageUri_fromPasteStr = function(pasteStr) {
	// Remove byte-order-markers if present, and continue...
	if (pasteStr.startsWith("\u{EF}\u{BB}\u{BF}")) // UTF-8 BOM
		pasteStr = pasteStr.replace("\u{EF}\u{BB}\u{BF}", "");
	if (pasteStr.startsWith("\u{FE}\u{FF}")) // UTF-16 BOM BE (Pasted from Linux-Firefox on Chromebook.)
		pasteStr = pasteStr.replace("\u{FE}\u{FF}", "");
	if (pasteStr.startsWith("\u{FF}\u{FE}")) // UTF-16 BOM LE
		pasteStr = pasteStr.replace("\u{FF}\u{FE}", "");
		
	if (pasteStr.startsWith("data:image/")) { // String is a dataUrl.
		return pasteStr;
	}
	if (pasteStr.startsWith("<")) { // It looks like html.
		var $paste = $(pasteStr);
		if (!$paste || $paste.length == 0) { return false; }
		var $img = false;
		if ($paste.get(0).nodeName.toLowerCase == "img") { // Pasted html is <img> tag (e.g. chromebook screenshot copy yields <img src="data:..."/>)
			$img = $paste;
		}
		else if ($paste.find("img").length) { // Pasted html contains inner-<img> (e.g. Firefox on ____ "<html><body> <img/> </body></html>")
			$img = $paste.find("img").first();
		}
		else { return false; } // looks like html, but no <img> tag found.
		
		return $img.attr("src");
	}

}

// Read/load one item from dataTransfer.  (Note, e.g. clipboard data disappears after getAsString() and getAsSFile(), so it's read-once.)
// "dataTransferItem" https://developer.mozilla.org/en-US/docs/Web/API/DataTransferItem
// - Note, getAsString() can only be called once, and clipboard/dataTransfer data is cleared!
mdImgImporter.prototype._imageUri_fromDataItem = function(dataXferItem, callback) {
	var self = this;
	var metaData = { kind:dataXferItem.kind, type:dataXferItem.type };
	if (dataXferItem.kind == "string") {
		dataXferItem.getAsString(function(strVal) { // Calling getAsString() seems to clear items array (length=0).  Undocumented?
			callback(self._imageUri_fromPasteStr(strVal), metaData);
		});
	}
	else if (dataXferItem.kind == "file") {
		var fileBlob = dataXferItem.getAsFile();
		var reader  = new FileReader();
		reader.readAsDataURL(fileBlob);
		reader.onload = function () {
			callback(reader.result, metaData); // return dataUrl.
		};
		reader.onerror = function (err) {
			g_genUI.transientMsg("error-file-to-data", "dataUrl-conversion-error: " + JSON.stringify(err));
		};
	}
	else { callback(false, metaData); }
}

// Helper function to avoid array-search clutter.
mdImgImporter.prototype._itemByKindTpye = function(items, kind, type) {
	for (var i=0; i<items.length; i++) {
		var item = items[i];
		if (item.kind.toLowerCase() == kind.toLowerCase() && item.type.toLowerCase() == type.toLowerCase()) { return item; }
	}
	return false;
}

// Dig in "DataTransfer" object (from "dragend" event and "paste" event), for a imageUri.
// - https://developer.mozilla.org/en-US/docs/Web/API/DataTransfer
mdImgImporter.prototype._imageUri_fromDataXfer = function(dataXferObj, callback) {
	var items = dataXferObj.items;
	//console.log("items.length: " + items.length);
	for (var i=0; i<items.length; i++) {
		var item = items[i];
		console.log("Pasted(" + i + "): " + item.kind + ", " + item.type);
	}

	var item = undefined;
	item = this._itemByKindTpye(items, "file", "image/png")
		|| this._itemByKindTpye(items, "file", "image/jpeg")
		|| this._itemByKindTpye(items, "file", "image/jpg")
		|| this._itemByKindTpye(items, "file", "image/bmp")
		|| this._itemByKindTpye(items, "string", "text/html")
		|| this._itemByKindTpye(items, "string", "text/plain");
	console.log("Trying item: " + item.kind + ", " + item.type);
	this._imageUri_fromDataItem(item, function(imageUri, tryInfo) {
		//if (imageUri) { console.log("imageUri result: " + imageUri.substring(0,400)); }
		callback(imageUri, tryInfo);
	});
}



mdImgImporter.prototype.wireupHandlers = function(callback) {
	var self = this;

	// Test-code for Android share intents (enabled with plugin "cc.fovea.cordova.openwith", in config.xml).
	//---------------------------------------------------------
	if (window.cordova && window.cordova.openwith) {
		// Import shared-in image.
		window.cordova.openwith.init(
			function() { // Success
				g_genUI.transientMsg("openwith-success", "openwith-success");
			}, 
			function() { // Error
				g_genUI.transientMsg("openwith-error", "openwith-error");
			}
		);

		window.cordova.openwith.addHandler(function(intent) {
			g_genUI.transientMsg("intent1", "intent1");
			g_genUI.transientMsg("intent-action", "action: " + intent.action);

			if (intent.items && intent.items.length) {
				var item = intent.items[0]; // assume 1 item
				g_genUI.transientMsg("intent-mime", item.type); // mime type
				g_genUI.transientMsg("intent-uri",  item.uri.substring(0,400)); // uri to the file, probably NOT a web uri ("content://com.someapp.../file.jpeg")
				//cordova.openwith.load(item, function(data, item) {
					// data is a long base64 string with the content of the file

				//});
				g_drawWrapper.setBackgroundPhoto(item.uri, function() {
					g_genUI.transientMsg("intent-DONE", "intent-DONE"); // mime type
				});
			}
		});
	}


	// Paste-plugin (test):
	// (enabled with cordova-clipboard plugin)
	//----------------------------------------
	if (window.cordova && window.cordova.plugins && window.cordova.plugins.clipboard) {

		var ctrlDown = false, ctrlKey = 17, cmdKey = 91, vKey = 86, cKey = 67, bKey = 66;
		$(document).on("keydown", function(e) {
			if (e.keyCode == ctrlKey || e.keyCode == cmdKey) ctrlDown = true;
		}).keyup(function(e) {
			if (e.keyCode == ctrlKey || e.keyCode == cmdKey) ctrlDown = false;
		});
		// Document Ctrl + C/V 
		$(document).on("keydown", function(e) {
			//if (ctrlDown && (e.keyCode == cKey)) console.log("Document catch Ctrl+C");
			if (ctrlDown && (e.keyCode == bKey)) { 
				g_genUI.transientMsg("ctrl-b", "Document catch Ctrl+b");
				cordova.plugins.clipboard.paste(function (text) {
					g_genUI.transientMsg("ctrl-b-text2", "Ctrl+b text(" + text.length + "): " + JSON.stringify(text).substring(0,400));
				});
			};
		});
	}

	// Direct (webview) Paste event
	//-----------------------------
	if (true) {
		$(document).on("paste", function(event) {
			var cbData = (event.clipboardData || event.originalEvent.clipboardData);

			self._imageUri_fromDataXfer(cbData, function(imageUri, tryInfo) {
				if (!imageUri) {
					g_genUI.transientMsg("paste-fail", "Image paste error: " + tryInfo.kind + ", " + tryInfo.type);
				}
				else {
					g_drawWrapper.setBackgroundPhoto(imageUri, function() {
						g_genUI.transientMsg("paste-success", "Image pasted: " + tryInfo.kind + ", " + tryInfo.type);
					});
				}
			});
			
		});
	}


	// Drag/drop:
	//----------------------------------------
	if (true) {
		document.addEventListener("drop", function(event) {
			event.preventDefault();
			$("#cnvsDraw").parent().css("border", "dashed 0px #0ff");

			var data = event.dataTransfer.getData('text/plain');
			var files = event.dataTransfer.files;
			console.log("drop-data (" + data.length + "):");
			console.log(data);
			console.log("drop-files: " + files.length);
			g_genUI.transientMsg("drop-files", "drop-files: " + files.length);
			g_genUI.transientMsg("drop-data-length", "drop-data-length (" + data.length + ")");
			g_genUI.transientMsg("drop-data-250", "drop-data-250: " + data.substring(0,250));
		});
		document.addEventListener("dragover", function(event) {
			event.preventDefault();
			//g_genUI.transientMsg("drag-over", "drag-over");
			$("#cnvsDraw").parent().css("border", "dashed 5px #0ff")
				.css("box-sizing", "border-box");
		});
		document.addEventListener("dragleave", function(event) {
			event.preventDefault();
			//g_genUI.transientMsg("drag-leave", "drag-leave");
			//alert(2);
			$("#cnvsDraw").parent().css("border", "dashed 0px #0ff");
		});
	}


}
