"use strict";

function mdPlaySound() {
	this.test = "";

	this.maxPitch = 11025;
}


mdPlaySound._playBufferData = function(bufferData) {
	var audioCtx = new AudioContext();

	// Home main gain (fade in gradually, because data has nasty habit of initial spike, and can't get gain node to work.)
	for (var i=0; i<1000; i++) {
		bufferData[i] *= i/1000;
	}
	
	var arrayBuffer = audioCtx.createBuffer(1, bufferData.length, 11025/*44100*/);
	var nowBuffering = arrayBuffer.getChannelData(0);
	for (var i = 0; i < arrayBuffer.length; i++) {
		nowBuffering[i] = bufferData[i];
	}

	var source = audioCtx.createBufferSource();
	source.buffer = arrayBuffer;
	//source.detune.value = -1200 * 2; // detune 2 octaves
	//source.playbackRate.value = 4; // play 4x (detune compensation) 
	source.connect(audioCtx.destination);

	//var gainNode = audioCtx.createGain();
	//gainNode.gain.setValueAtTime(0, audioCtx.currentTime);
	//gainNode.gain.linearRampToValueAtTime(1.0, audioCtx.currentTime + 50); // ramp in sound to prevent super loug glitch at start.
	//gainNode.gain.setValueAtTime(1, audioCtx.currentTime + 50);
	//source.connect(gainNode);
	//gainNode.connect(audioCtx.destination);
	source.start();
}

// Convert frequency amplitude data to time-domain (waveform)
// - FFT is its own inverse, apparently. (so reverse == fwd).
mdPlaySound._reverseFFT_1 = function(freqData) {
	var bufferSize = freqData.length;
	if (Math.log2(bufferSize) % 1 !== 0) { throw new Error("bufferSize not power of 2"); }
	var ffter = new FFTNayuki(bufferSize);
	var output = freqData.map(function(x) { return x; });
	ffter.forward(output, (new Array(bufferSize)).fill(0));
	return output;
}

mdPlaySound._reverseFFT_2 = function(freqData) {
	var bufferSize = freqData.length;
	if (Math.log2(bufferSize) % 1 !== 0) { throw new Error("bufferSize not power of 2"); }
	var ffter = new FFTNayuki(bufferSize);
	var real = freqData.map(function(x) { return x; });
	var imag = (new Array(bufferSize)).fill(0);
	ffter.forward(real, imag);
	var output = [];
	for (var i=0; i<bufferSize; i++) {
		output[i] = Math.pow(real[i], 2) + Math.pow(imag[i], 2);
		output[i] = Math.sqrt(output[i]);
	}
	return real;
}

mdPlaySound._bufferData_fromCnvs = function(cnvs) {
	//var cnvs = mdCanvasExport.getExportCnvs();
	var ctx = cnvs.getContext('2d');
	var output = [];

	var cnvs1px = document.createElement('canvas'); // 1px-wide staging cnvs
	cnvs1px.width = 1;
	cnvs1px.height = 1024; // Affects speed of playback (lower# == faster).  Also resolution! :(
	var ctx1px = cnvs1px.getContext('2d');


	// Loop thru each column of pixels.
	var speed = 4; // step sample window by this
	var sampleSize = 6; // sample window width (If larger than speed, windows overlap.)
	for (var i=0; i<cnvs.width/speed; i+=speed) {
		ctx1px.drawImage(cnvs, i*speed, 0, sampleSize, cnvs.height,
			0, 0, cnvs1px.width/*1*/, cnvs1px.height);
		
		// Get the CanvasPixelArray from the given coordinates and dimensions.
		var pix = ctx1px.getImageData(0, 0, 1, cnvs1px.height).data;

		var freqData = [];
		for (var j=0, n=pix.length; j<n; j+=4) {
			var value = (pix[j] + pix[j+1] + pix[j+2]) / 3; // (r+b+g)/3 = avg lightness.
			// i+3 is alpha (the fourth element)
			freqData[Math.round(j/4)] = 255-value; // 255-x so "white" background is silence.
		}

		// fill the upper range of frequencies w/ silence, so freqData gets pushed lower into interesting-audible range.
		freqData = (new Array(freqData.length*3)).fill(0).concat(freqData);

		var bufferData = mdPlaySound._reverseFFT_2(freqData); // waveForm
		//console.log(bufferData);
		output = output.concat(bufferData);
	}
	console.log("output.length: " + output.length);

	return output;
}

mdPlaySound.prototype.wireupHandlers = function() {
	var self = this;
	
	// console.log() test of what FFT does to data.
	$(".jsAppBody").on("click", ".jsPlaySound_test1", function(ev) {
		var bufferSize = 64;
		var ffter = new FFTNayuki(bufferSize);
		var valsFFT = (new Array(bufferSize)).fill(0);
		valsFFT[3] = 254;
		console.log("vals1:" + JSON.stringify(valsFFT));
		ffter.forward(valsFFT, (new Array(bufferSize)).fill(0));
		console.log("vals2:" + JSON.stringify(valsFFT.map(function(x) { return Math.round(x*10) / 10; })));

		mdPlaySound._playBufferData(valsFFT);
	});

	$(".jsAppBody").on("click", ".jsPlaySound_test2", function(ev) {
		var valsFreq = (new Array(1024)).fill(0);
		valsFreq[30] = 254;
		console.log("vals1:" + JSON.stringify(valsFreq));
		var valsWave = mdPlaySound._reverseFFT_1(valsFreq);
		console.log("vals2:" + JSON.stringify(valsWave.map(function(x) { return Math.round(x*10) / 10; })));

		mdPlaySound._playBufferData(valsWave);
	});
	

	$(".jsAppBody").on("click", ".jsPlaySound_test3", function(ev) {
		var cnvs = mdCanvasExport.getExportCnvs();
		var bufferData = mdPlaySound._bufferData_fromCnvs(cnvs);
		mdPlaySound._playBufferData(bufferData);
	});
}


// Temporary, for testing.
var g_playSound = new mdPlaySound();
g_playSound.wireupHandlers();
